/***************************************************************************
                          Vocabulary.cpp  -  description
                             -------------------
    begin                : Fri Jan 5 2001
    copyright            : (C) 2001 by Sauro Menchetti
    email                : menco@inwind.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Vocabulary.cpp

#include "Vocabulary.h"
#include "HTMLPage.h"
#include <cmath>

void Vocabulary::init(std::string mode)
{
    if (mode == "r")
    {
        readVocabulary();
    }
    else if (mode == "w")
    {
        createVocabulary();
        writeVocabulary();
    }
    else
    {
        std::cerr << "Vocabulry mode not allowed" << std::endl;
        assert(1 != 1);
    }
}

void Vocabulary::readVocabulary()
{
	std::ifstream is(vocabularyFileName.c_str());

	if (!is.is_open())
	{
		std::cerr << "Can't open the file " << vocabularyFileName << std::endl;		
		exit(-1);
	}

	// vocabulary word read from file
	std::string word;

	// read vocabulary word from file and insert it in vocabulary vector
	while(std::getline(is, word) != 0)
	{
        // add word to vocabulary vector
        vocabulary_vec.push_back(word);
	}

	is.close();

    // vocabulary words' number
    unsigned int vocabulary_vec_size = vocabulary_vec.size();

    // add words to vocabulary map and set the word index
    for (unsigned int i = 0; i < vocabulary_vec_size; ++i)
    {
        vocabulary_map[vocabulary_vec[i]] = i;
    }
}

void Vocabulary::createVocabulary()
{
	// open the index file
	std::ifstream is(indexFileName.c_str());

	if (!is.is_open())
	{
		std::cerr << "Can't open the file " << indexFileName << std::endl;		
		exit(-1);
	}

	// file name of html page
	std::string pageFileName;

	// page number
	unsigned int pagenumber = 0;

	// read vocabulary word from file and insert it in vocabulary vector
	while(std::getline(is, pageFileName) != 0)
	{
		std::cout << "page number: " << ++pagenumber << " " << pageFileName << std::endl;

		// create the html page
		HTMLPage htmlpage(pageFileName);

//std::cout << "CREATE" << std::endl << htmlpage << std::endl; std::cin.get();

		// processing the html page for create the vocabulary
		htmlpage.processing_for_create_vocabulary(stopWordsFileName);

//std::cout << "HTML PAGE AFTER PROCESSING" << std::endl << htmlpage << std::endl;  std::cin.get();

		// html page split in token
		std::vector<std::string> tokenVector = htmlpage.getTokenVector();

		// number of token of html page
		unsigned int token_number = tokenVector.size();

		// map of token of html page: useful for IDF (inverse document frequency)
		std::map<std::string, unsigned int> token_map;

		for (unsigned int ti = 0; ti < token_number; ++ti)
		{
    	    token_map[tokenVector[ti]]++;
		}

		std::map<std::string, unsigned int>::const_iterator it_token_map;

		for (it_token_map = token_map.begin(); it_token_map != token_map.end(); ++it_token_map)
		{
			// insert the word in the vocabulary map
    	    vocabulary_map[it_token_map->first]++;
		}
	
		// in this way, vocabulary_map.second is DF(wi)

		/////////////////////////////////////////////////////////////
		// we can remove vocabulary word from frequency if we want //
		/////////////////////////////////////////////////////////////

		//const unsigned int min_word_frequency = 10;
		//const unsigned int max_word_frequency = 1000;

		//eraseWord(min_word_frequency, max_word_frequency)
	}

    std::map<std::string, unsigned int>::const_iterator it_vocabulary_map;

	/////////////////////////////////////////////////////////////////////////////////
	// vector of IDF (inverse document frequency) IDF(wi) = log (|D| / DF(wi)), where
	// |D| is number of documents (html pages in this case)
	// DF(wi) is number of documents (html pages in this case) where there is word wi
	/////////////////////////////////////////////////////////////////////////////////
	std::vector<double> vector_idf;

	for(it_vocabulary_map = vocabulary_map.begin(); it_vocabulary_map != vocabulary_map.end(); ++it_vocabulary_map)
	{
		// inverse document frequency
		double idf = log(static_cast<double>(pagenumber) / it_vocabulary_map->second);
		vector_idf.push_back(idf);
	}

	// open the idf file
	std::ofstream os_idf("./inverse_document_frequency");

	if (!os_idf.is_open())
	{
		std::cerr << "Can't open the file ./inverse_document_frequency" << std::endl;		
		exit(-1);
	}

	unsigned int vector_idf_size = vector_idf.size();

	for (unsigned int i = 0; i < vector_idf_size; ++i)
	{
		os_idf << vector_idf[i] << std::endl;
	}

	// close the idf file
	os_idf.close();

	// close the index file
	is.close();
}

void Vocabulary::writeVocabulary()
{
	std::ofstream os(vocabularyFileName.c_str());

	if (!os.is_open())
	{
		std::cerr << "Can't open the file " << vocabularyFileName << std::endl;		
		exit(-1);
	}

    std::map<std::string, unsigned int>::const_iterator it_vocabulary_map;

    for (it_vocabulary_map = vocabulary_map.begin(); it_vocabulary_map != vocabulary_map.end(); ++it_vocabulary_map)
    {
        os << it_vocabulary_map->first << std::endl;
    }

	os.close();
}

std::ostream& operator<<(std::ostream& os, const Vocabulary& vocabulary)
{
    os << "VOCABULARY" << std::endl << std::endl;

    os << "Vocabulary File Name: " << vocabulary.getVocabularyFileName() << std::endl;
    os << "Index File Name: "      << vocabulary.getIndexFileName() << std::endl;
    os << "Stop Words File Name: " << vocabulary.getStopWordsFileName() << std::endl;

    os << "Vocabulary Vector" << std::endl;

    std::vector<std::string> voc_vec = vocabulary.getVocabulary_vec();
    unsigned int voc_vec_size = voc_vec.size();

    for (unsigned int i = 0; i < voc_vec_size; ++i)
    {
        os << i << ") " << voc_vec[i] << std::endl;
    }

    os << "Vocabulary Map<Word, Index>" << std::endl;

    std::map<std::string, unsigned int> voc_map = vocabulary.getVocabulary_map();
    std::map<std::string, unsigned int>::const_iterator it_voc_map;

    for (it_voc_map = voc_map.begin(); it_voc_map != voc_map.end(); ++it_voc_map)
    {
        os << it_voc_map->second << ") " << it_voc_map->first << std::endl;
    }

    return os;
}
